Module: MangoPay

Defined in:
lib/mangopay/json.rb,
lib/mangopay.rb,
lib/mangopay/ubo.rb,
lib/mangopay/card.rb,
lib/mangopay/hook.rb,
lib/mangopay/user.rb,
lib/mangopay/event.rb,
lib/mangopay/client.rb,
lib/mangopay/errors.rb,
lib/mangopay/pay_in.rb,
lib/mangopay/refund.rb,
lib/mangopay/report.rb,
lib/mangopay/wallet.rb,
lib/mangopay/deposit.rb,
lib/mangopay/dispute.rb,
lib/mangopay/mandate.rb,
lib/mangopay/pay_out.rb,
lib/mangopay/version.rb,
lib/mangopay/resource.rb,
lib/mangopay/transfer.rb,
lib/mangopay/acquiring.rb,
lib/mangopay/recipient.rb,
lib/mangopay/report_v2.rb,
lib/mangopay/conversion.rb,
lib/mangopay/http_calls.rb,
lib/mangopay/legal_user.rb,
lib/mangopay/regulatory.rb,
lib/mangopay/settlement.rb,
lib/mangopay/transaction.rb,
lib/mangopay/bank_account.rb,
lib/mangopay/kyc_document.rb,
lib/mangopay/natural_user.rb,
lib/mangopay/bankingaliases.rb,
lib/mangopay/legal_user_sca.rb,
lib/mangopay/ubo_declaration.rb,
lib/mangopay/virtual_account.rb,
lib/mangopay/natural_user_sca.rb,
lib/mangopay/card_registration.rb,
lib/mangopay/filter_parameters.rb,
lib/mangopay/pre_authorization.rb,
lib/mangopay/authorization_token.rb,
lib/mangopay/bankingaliases_iban.rb,
lib/mangopay/identity_verification.rb,
lib/mangopay/payment_method_metadata.rb

Overview

We can use MultiJson directly , why do we even have this module ?

Defined Under Namespace

Modules: AuthorizationToken, FilterParameters, HTTPCalls, JSON Classes: Acquiring, BankAccount, BankingAliases, BankingAliasesIBAN, Card, CardRegistration, Client, Configuration, Conversion, Deposit, Dispute, Error, Event, Hook, IdentityVerification, KycDocument, LegalUser, LegalUserSca, Mandate, NaturalUser, NaturalUserSca, PayIn, PayOut, PaymentMethodMetadata, PreAuthorization, Recipient, Refund, Regulatory, Report, ReportV2, Resource, ResponseError, Settlement, Transaction, Transfer, Ubo, UboDeclaration, User, VirtualAccount, Wallet

Constant Summary collapse

VERSION =
'3.49.0'

Class Method Summary collapse

Class Method Details

.add_config(name, config) ⇒ Object

Add MangoPay.Configuration to the list of configs



194
195
196
# File 'lib/mangopay.rb', line 194

def add_config(name, config)
  @configurations[name] = config
end

.api_pathObject



141
142
143
# File 'lib/mangopay.rb', line 141

def api_path
  "/#{version_code}/#{MangoPay.configuration.client_id}"
end

.api_path_no_clientObject



145
146
147
# File 'lib/mangopay.rb', line 145

def api_path_no_client
  "/#{version_code}"
end

.api_path_v3Object



149
150
151
# File 'lib/mangopay.rb', line 149

def api_path_v3
  "/#{version_code_v3}/#{MangoPay.configuration.client_id}"
end

.api_uri(url = '') ⇒ Object



153
154
155
# File 'lib/mangopay.rb', line 153

def api_uri(url='')
  URI(configuration.root_url + url)
end

.configurationObject



162
163
164
165
166
167
168
169
# File 'lib/mangopay.rb', line 162

def configuration
  config = Thread.current[:mangopay_configuration]

  config                                                ||
  ( @last_configuration_set &&
    (self.configuration = @last_configuration_set.dup)) ||
  (self.configuration = MangoPay::Configuration.new)
end

.configuration=(value) ⇒ Object



157
158
159
160
# File 'lib/mangopay.rb', line 157

def configuration=(value)
  Thread.current[:mangopay_configuration] = value
  @last_configuration_set = value
end

.configure {|config| ... } ⇒ Object

Yields:

  • (config)


171
172
173
174
175
# File 'lib/mangopay.rb', line 171

def configure
  config = self.configuration
  yield config
  self.configuration = config
end

.fetch_response(idempotency_key) ⇒ Object

Retrieve a previous response by idempotency_key See docs.mangopay.com/api-references/idempotency-support/



342
343
344
345
# File 'lib/mangopay.rb', line 342

def fetch_response(idempotency_key)
  url = "#{api_path}/responses/#{idempotency_key}"
  request(:get, url)
end

.get_config(name) ⇒ Object

Fetch a MangoPay configuration from the list of configs. Throw error if not found



199
200
201
202
203
# File 'lib/mangopay.rb', line 199

def get_config(name)
  config = @configurations[name]
  raise "Could not find any configuration with name '#{name}'" unless config
  config
end

.ratelimitObject



185
186
187
# File 'lib/mangopay.rb', line 185

def ratelimit
  @ratelimit
end

.ratelimit=(obj) ⇒ Object



189
190
191
# File 'lib/mangopay.rb', line 189

def ratelimit=(obj)
  @ratelimit = obj
end

.remove_config(name) ⇒ Object



205
206
207
208
# File 'lib/mangopay.rb', line 205

def remove_config(name)
  raise "Could not find any configuration with name '#{name}'" unless @configurations[name]
  @configurations[name] = nil
end

.request(method, url, params = {}, filters = {}, headers_or_idempotency_key = nil, before_request_proc = nil) ⇒ Object

  • method: HTTP method; lowercase symbol, e.g. :get, :post etc.

  • url: the part after Configuration#root_url

  • params: hash; entity data for creation, update etc.; will dump it by JSON and assign to Net::HTTPRequest#body

  • filters: hash; pagination params etc.; will encode it by URI and assign to URI#query

  • headers_or_idempotency_key: hash of headers; or replaced by request_headers if nil; or added to request_headers as idempotency key otherwise (see docs.mangopay.com/api-references/idempotency-support/)

  • before_request_proc: optional proc; will call it passing the Net::HTTPRequest instance just before Net::HTTPRequest#request

Raises MangoPay::ResponseError if response code != 200.



220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
# File 'lib/mangopay.rb', line 220

def request(method, url, params={}, filters={}, headers_or_idempotency_key = nil, before_request_proc = nil)
  uri = api_uri(url)
  uri.query = URI.encode_www_form(filters) unless filters.empty?

  if headers_or_idempotency_key.is_a?(Hash)
    headers = headers_or_idempotency_key
  else
    headers = request_headers
    headers['Idempotency-Key'] = headers_or_idempotency_key if headers_or_idempotency_key != nil
  end

  if configuration.uk_header_flag
    headers['x-tenant-id'] = 'uk'
  end

  res = Net::HTTP.start(uri.host, uri.port, **ssl_options) do |http|
    req = Net::HTTP::const_get(method.capitalize).new(uri.request_uri, headers)
    req.body = JSON.dump(params)
    before_request_proc.call(req) if before_request_proc
    do_request(http, req, uri)
  end

  raise MangoPay::ResponseError.new(uri, '408', {'Message' => 'Request Timeout'}) if res.nil?

  # decode json data
  begin
    data = res.body.to_s.empty? ? {} : JSON.load(res.body.to_s)
  rescue MultiJson::ParseError
    details = {}
    details['Message'] = res.body
    raise MangoPay::ResponseError.new(uri, res.code, details)
  end

  unless res.is_a?(Net::HTTPOK) or res.is_a?(Net::HTTPCreated) or res.is_a?(Net::HTTPNoContent)
    if res.is_a?(Net::HTTPUnauthorized) and res['www-authenticate'] != nil
      redirect_url = res['www-authenticate'][/RedirectUrl=(.+)/, 1]
      data['RedirectUrl'] = redirect_url
      data['message'] = 'SCA required to perform this action.'
      raise MangoPay::ResponseError.new(uri, res.code, data)
    end
    raise MangoPay::ResponseError.new(uri, res.code, data)
  end

  # copy pagination info if any
  ['x-number-of-pages', 'x-number-of-items'].each { |k|
    filters[k.gsub('x-number-of-', 'total_')] = res[k].to_i if res[k]
  }

  if res['x-ratelimit']
    self.ratelimit = {
      limit: res['x-ratelimit'].split(", "),
      remaining: res['x-ratelimit-remaining'].split(", "),
      reset: res['x-ratelimit-reset'].split(", ")
    }
  end

  configuration.after_request_proc&.call(data)

  data
end

.request_multipart(method, url, file, file_name, idempotency_key = nil) ⇒ Object



281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
# File 'lib/mangopay.rb', line 281

def request_multipart(method, url, file, file_name, idempotency_key = nil)
  uri = api_uri(url)

  # Set headers
  headers = request_headers
  headers['Idempotency-Key'] = idempotency_key if idempotency_key
  headers['x-tenant-id'] = 'uk' if configuration.uk_header_flag

  boundary = "#{SecureRandom.hex}"
  headers['Content-Type'] = "multipart/form-data; boundary=#{boundary}"

  multipart_body = []
  multipart_body << "--#{boundary}"
  multipart_body << "Content-Disposition: form-data; name=\"file\"; filename=\"#{file_name}\""
  multipart_body << ""
  multipart_body << file
  multipart_body << "--#{boundary}--"
  multipart_body << ""

  # Join string parts and ensure binary content is preserved
  body = multipart_body.map { |part| part.is_a?(String) ? part.force_encoding("ASCII-8BIT") : part }.join("\r\n")

  res = Net::HTTP.start(uri.host, uri.port, **ssl_options) do |http|
    req_class = Net::HTTP.const_get(method.capitalize)
    req = req_class.new(uri.request_uri, headers)
    req.body = body
    do_request(http, req, uri)
  end

  raise MangoPay::ResponseError.new(uri, '408', { 'Message' => 'Request Timeout' }) if res.nil?

  begin
    data = res.body.to_s.empty? ? {} : JSON.load(res.body.to_s)
  rescue MultiJson::ParseError
    raise MangoPay::ResponseError.new(uri, res.code, { 'Message' => res.body })
  end

  unless res.is_a?(Net::HTTPOK) || res.is_a?(Net::HTTPCreated) || res.is_a?(Net::HTTPNoContent)
    if res.is_a?(Net::HTTPUnauthorized) && res['www-authenticate']
      redirect_url = res['www-authenticate'][/RedirectUrl=(.+)/, 1]
      data['RedirectUrl'] = redirect_url
      data['message'] = 'SCA required to perform this action.'
    end
    raise MangoPay::ResponseError.new(uri, res.code, data)
  end

  if res['x-ratelimit']
    self.ratelimit = {
      limit: res['x-ratelimit'].split(", "),
      remaining: res['x-ratelimit-remaining'].split(", "),
      reset: res['x-ratelimit-reset'].split(", ")
    }
  end

  configuration.after_request_proc&.call(data)

  data
end

.version_codeObject



133
134
135
# File 'lib/mangopay.rb', line 133

def version_code
  "v2.01"
end

.version_code_v3Object



137
138
139
# File 'lib/mangopay.rb', line 137

def version_code_v3
  "V3.0"
end

.with_configuration(config) ⇒ Object



177
178
179
180
181
182
183
# File 'lib/mangopay.rb', line 177

def with_configuration(config)
  original_config = MangoPay.configuration
  MangoPay.configuration = config
  yield
ensure
  MangoPay.configuration = original_config
end